#!/usr/bin/env python
#
# Copyright (C) 2009-2013  Kipp Cannon, Chad Hanna, Drew Keppel
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

### A program to produce a variety of plots from a gstlal inspiral analysis, e.g. IFAR plots, missed found, etc.

#
# =============================================================================
#
#                                   Preamble
#
# =============================================================================
#


import bisect
import itertools
import json
import math
import matplotlib
import matplotlib.figure
matplotlib.rcParams.update({
	"font.size": 12.0,
	"axes.titlesize": 12.0,
	"axes.labelsize": 12.0,
	"xtick.labelsize": 12.0,
	"ytick.labelsize": 12.0,
	"legend.fontsize": 10.0,
	"figure.dpi": 300,
	"savefig.dpi": 300,
	"text.usetex": True,
	"path.simplify": True,
	"font.family": "serif"
})
from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
import numpy
import operator
from optparse import OptionParser
import os
import scipy
import sqlite3
import sys
import matplotlib.ticker

golden_ratio = (1. + math.sqrt(5.))

def format_column(column):
	return column.lower().replace(' ', '_').replace(':', '').replace(';', '')


def save_table(fname, fields, rows):
	### format data
	fields = [{'key': format_column(col), 'label': col, 'sortable': True} for col in fields]
	colnames = [field['key'] for field in fields]
	items = [{col: val for col, val in zip(colnames, row)} for row in rows]

	### write to disk
	with open(fname, 'w') as f:
		f.write(json.dumps({'fields': fields, 'items': items}))

def roman(i, arabics = (1000,900,500,400,100,90,50,40,10,9,5,4,1), romans = ("m","cm","d","cd","c","xc","l","xl","x","ix","v","iv","i")):
	if not arabics:
		return ""
	if i < arabics[0]:
		return roman(i, arabics[1:], romans[1:])
	return romans[0] + roman(i - arabics[0], arabics, romans)

def truncate(f, digits, si=True):
	assert digits > 0, 'number of digits must be positive'
	if si:
		return float(('%10.' + str(digits) + 'e') % f)
	else:
		return float(('%10.' + str(digits) + 'f') % f)

def create_plot(x_label = None, y_label = None, width = 165.0, aspect = golden_ratio):
	fig = matplotlib.figure.Figure()
	FigureCanvas(fig)
	if aspect is None:
		aspect = golden_ratio
	fig.set_size_inches(6.49606299, 4.01478772)
	print(width / 25.4, width / 25.4 / aspect)
	axes = fig.add_axes((0.1, 0.12, .875, .80))
	axes.grid(True)
	if x_label is not None:
		axes.set_xlabel(x_label)
	if y_label is not None:
		axes.set_ylabel(y_label)
	return fig, axes

def sigma_region(mean, nsigma):
	return numpy.concatenate((mean - nsigma * numpy.sqrt(mean), (mean + nsigma * numpy.sqrt(mean))[::-1]))

# Create figure
fig, axes = create_plot(r"Inverse False-Alarm Rate (yr)", r"Cumulative Number of Events")
axes.loglog()
axes.tick_params(axis='both', which='both')
axes.grid(axis='both', linestyle=':', c='k')
locmaj = matplotlib.ticker.LogLocator(base=10,numticks=12) 
axes.xaxis.set_major_locator(locmaj)
locmin = matplotlib.ticker.LogLocator(base=10.0, subs=numpy.arange(2, 10) * .1,numticks=100)
axes.xaxis.set_minor_locator(locmin)
axes.xaxis.set_minor_formatter(matplotlib.ticker.NullFormatter())
xlim = numpy.load("gstlal_figure_xlim.npy")
ylim = numpy.load("gstlal_figure_ylim.npy")
expected_count_x = numpy.load("gstlal_figure_expected_count_x.npy")
expected_count_x_for_sigma = numpy.load("gstlal_figure_expected_count_x_for_sigma.npy")
expected_count_y = numpy.load("gstlal_figure_expected_count_y.npy")
one_sigma_region_y = numpy.load("gstlal_figure_1_sigma_y.npy")
two_sigma_region_y = numpy.load("gstlal_figure_2_sigma_y.npy")
three_sigma_region_y = numpy.load("gstlal_figure_3_sigma_y.npy")
line1, = axes.plot(expected_count_x, expected_count_y, 'k--', linewidth = 1)
line2, = axes.fill(expected_count_x_for_sigma, three_sigma_region_y, alpha = 0.5, facecolor = [0.75, 0.75, 0.75])
line3, = axes.fill(expected_count_x_for_sigma, two_sigma_region_y, alpha = 0.3, facecolor = [0.5, 0.5, 0.5])
line4, = axes.fill(expected_count_x_for_sigma, one_sigma_region_y, alpha = 0.3, facecolor = [0.25, 0.25, 0.25])
zerolag_stats_x = numpy.load("gstlal_figure_zerolag_stats_x.npy")
zerolag_stats_y = numpy.load("gstlal_figure_zerolag_stats_y.npy")
line5, = axes.plot(zerolag_stats_x, zerolag_stats_y, 'k', linewidth = 2)
axes.legend((line5, line1, line4, line3, line2), ("Observed", r"Expected background", r"$\pm\sigma$", r"$\pm 2\sigma$", r"$\pm 3\sigma$"), loc = "upper right")
axes.set_xlim(xlim)
axes.set_ylim(ylim)
if fig is not None:
	try:
		fig.tight_layout(rect=(0.03, 0.02, 0.98, 0.98))
	except ValueError:
		pass
fig.savefig("GstLAL_subthreshold_lensing_figure_4.pdf")
